#include <p32xxxx.h>
#include <plib.h>
#include "PIC32MX120pins.h"

// DEVCFG3:
#pragma config IOL1WAY  = OFF       // Peripheral Pin Select Configuration, allow mult reconfig
#pragma config PMDL1WAY = OFF				// Peripheral Module Disable Config, allow mult reconfig
// DEVCFG2:
#pragma config FPLLODIV = DIV_2     // PLL Output Divider
#pragma config FPLLMUL  = MUL_20    // PLL Multiplier
#pragma config FPLLIDIV = DIV_2     // PLL Input Divider
// 8MHz / 2 * 20 / 2 = 40MHz
// DEVCFG1:
#pragma config FWDTEN   = OFF       // Watchdog Timer
#pragma config WDTPS    = PS1       // Watchdog Timer Postscale
#pragma config FCKSM    = CSDCMD    // Clock Switching & Fail Safe Clock Monitor
#pragma config FPBDIV   = DIV_1     // Peripheral Clock divisor
#pragma config OSCIOFNC = OFF       // CLKO Enable
#pragma config POSCMOD  = OFF       // Primary Oscillator
#pragma config IESO     = OFF       // Internal/External Switch-over
#pragma config FSOSCEN  = OFF       // Secondary Oscillator Enable (KLO was off)
#pragma config FNOSC    = FRCPLL    // Oscillator Selection
// DEVCFG0:
#pragma config CP       = OFF       // Code Protect
#pragma config BWP      = ON        // Boot Flash Write Protect
#pragma config PWP      = OFF       // Program Flash Write Protect
#pragma config ICESEL   = ICS_PGx1  // ICE/ICD Comm Channel Select
#pragma config JTAGEN   = OFF       // JTAG Enable
#pragma config DEBUG    = OFF       // Background Debugger Enable

#define SYS_FREQ 40000000L

int main(void) {

	CFGCONbits.JTAGEN = 0; // turn off JTAG, get back those pins for IO use

	SYSTEMConfigPerformance(SYS_FREQ);
	INTEnableSystemMultiVectoredInt();

	// Set B15 (pin 26) to OC1, A0 (pin 2) to AN0, everything else digital input
	PIC32MX120_setup_pins();

	// set up OC1 to 1kHz PWM
	T2CON = 0; // turn off TMR2
	OC1CON = 0; //turn off OC1
	T2CONbits.TCKPS = 2; // set prescaler 1:4
	TMR2 = 0; // start TMR2 counting from zero.
	PR2 = 9999; // set period match value
	/* so 1kHz from 9999 from (40MHz / 4 prescaler / 1000kHz) - 1 = 9999 */
	OC1CONbits.OCM = 0b110; // enable PWM mode with no fault protection
	OC1RS = 5000;
	OC1R = 5000;
	T2CONbits.ON = 1; // turn on TMR2
	OC1CONbits.ON = 1; // turn OC1 back on

	// set up ADC to read AN0, AN0 already set to pin A0
	AD1CON3bits.ADCS = 3; // ADC clock period is Tad = 2 * ADCS * Tpb
	AD1CHSbits.CH0SA = 0; // AN0 is selected
	AD1CON1bits.ADON = 1; // turn on A/D converter

	// some variables for the analog conversion
	int elapsed = 0;
	int finishtime = 0;
	int sampletime = 20;
	int analogValue = 0;

	while (1) {
		// if a button on B14 (pin 25) is pushed, turn on LED at B15 to
		// brightness cooresponding to the potentiometer at A0
		// otherwise turn it off
		if (PORTBbits.RB14) {
			WriteCoreTimer(0);
			AD1CON1bits.SAMP = 1; // start sampling
			elapsed = ReadCoreTimer();
			finishtime = elapsed + sampletime;
			while (ReadCoreTimer() < finishtime); // sample for more than 200 ns
			AD1CON1bits.SAMP = 0; // stop sampling and start converting
			while (!AD1CON1bits.DONE); // wait for the conversion process to finish
			analogValue = ADC1BUF0; // read the buffer with the result
			OC1RS = analogValue*9999/1023; // scale it from 2^10 to 9999
		} else {
			OC1RS = 0;
		}
	}

	return 0;
}
